/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <util/crc16.h>
#ifdef WATCHDOG
#include <avr/wdt.h>
#endif

#include <1wire/1wire-config.h>
#include <1wire/1wire-error.h>
#include <1wire/1wire.h>
#include <1wire/1wire-devices.h>
#include <1wire/1wire-manage.h>

w1_dev *w1_root=NULL;
unsigned int romcount=0;

//allocate memory and call chip-init-functions
unsigned char chip_init(unsigned char bus_nr, w1_dev *rom)
{
	switch (rom->id[0]) {
		//temperatur
		case CHIP_DS18B20:
		case CHIP_DS18S20:
			if (!rom->value) rom->value = calloc(1,sizeof(ds18x20_value));
			if (!rom->value) return 0;				//alloc failed
			ds18x20_init(bus_nr, rom);					//device-specific init
			break;
	}
	return 1;
}


w1_dev *find_rom(unsigned char *id)
{
	unsigned char cnt, found;
	w1_dev *rom;

	for (rom=w1_root;rom;rom=rom->next) {
		found = 1;
		for (cnt=0;cnt<8;cnt++) {
			if (rom->id[cnt] != id[cnt]) {
				found = 0;
				break;
			}
		}
		if (found) return rom;
	}
	return NULL;
}


//allocate memory and initialize w1_dev structure
w1_dev *create_rom(unsigned char bus_nr, unsigned char *id)
{
	w1_dev *rom;

	rom = find_rom(id);										//rom already in list?
	if (!rom) {
		rom = calloc(1,sizeof(w1_dev));
		if (!rom) return NULL;								//alloc failed
		memcpy(rom->id,id,8);								//copy id
		rom->next = w1_root;								//insert at top of list
		w1_root = rom;
		romcount++;
	}
	rom->flags = bus_nr;
	rom->err = NO_ERR;
	if (!chip_init(bus_nr, rom)) {
		w1_root = rom->next;								//remove element
		free(rom);
		return NULL;
	}
	return rom;
}

//find roms an a specific bus
unsigned char search_roms(unsigned char bus_nr, unsigned char master_reset)
{
	unsigned char err, cnt, id[8], crc8;
	unsigned long long w1sseq;

	for (w1sseq=SEARCH_FIRST;w1sseq!=LAST_DEVICE;) {
		err = w1_reset(bus_nr, 0);									//error, no device found
		if (err) return err;
		err = w1_rom_search(bus_nr, &w1sseq, id);
		if (err) return err;
		crc8 = 0;
		for (cnt=0;cnt<7;cnt++) {
			crc8 = _crc_ibutton_update(crc8, id[cnt]);
		}
		if (crc8 != id[7]) return CRC_ERR;
		if (!create_rom(bus_nr, id)) return OUT_OF_MEM_ERR;
	}
	return 0;
}
