/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <lcd-alpha.h>
#include <util/delay.h>


void LCD_byte_out(unsigned char rs, unsigned char busy, unsigned char data)
{
	// port X output
	LCD_DATA_DDR |= 0x0F<<LCD_DATA_BIT;
	//set rs
	LCD_RS_PORT &= ~_BV(LCD_RS_BIT);
	LCD_RS_PORT |= rs<<LCD_RS_BIT;
	//set write
	LCD_RW_PORT &= ~_BV(LCD_RW_BIT);
	// write 1st nibble
	LCD_DATA_PORT &= 0xF0;
	LCD_DATA_PORT |= (data & 0xF0)>>4;
	LCD_E_PORT |= _BV(LCD_E_BIT);
	LCD_E_PORT &= ~_BV(LCD_E_BIT);
	//write 2nd nibble
	LCD_DATA_PORT &= 0xF0;
	LCD_DATA_PORT |= data & 0x0F;
	LCD_E_PORT |= _BV(LCD_E_BIT);
	LCD_E_PORT &= ~_BV(LCD_E_BIT);
	LCD_wait_busy(busy);
}

unsigned char LCD_byte_in(unsigned char rs)
{
	unsigned char result;

	// port X input
	LCD_DATA_DDR &= ~(0x0F<<LCD_DATA_BIT);
	//set rs
	LCD_RS_PORT &= ~_BV(LCD_RS_BIT);
	LCD_RS_PORT |= rs<<LCD_RS_BIT;
	//set read
	LCD_RW_PORT |= _BV(LCD_RW_BIT);
	// read 1st nibble
	LCD_E_PORT |= _BV(LCD_E_BIT);
	nop(); nop(); nop(); nop();
	result = LCD_DATA_PIN<<4;
	LCD_E_PORT &= ~_BV(LCD_E_BIT);
	nop(); nop(); nop(); nop();
	//read 2nd nibble
	LCD_E_PORT |= _BV(LCD_E_BIT);
	nop(); nop(); nop(); nop();
	result |= LCD_DATA_PIN & 0x0F;
	LCD_E_PORT &= ~_BV(LCD_E_BIT);
	return result;
}


void LCD_wait_busy(unsigned char busy)
{
	if (busy) {
		while (LCD_byte_in(0) & 0x80) {};
	} else {
		_delay_ms(LCD_LONGDELAY);
	}
}


void LCD_init(void)
{
	unsigned char init_sequence[9] = { 0x33, 0x33, 0x32, 0x28, 6, 0x0E, 0x1C, 1, 2 };
	unsigned char cnt;

	//control bits -> output
	LCD_RS_DDR |= _BV(LCD_RS_BIT);
	LCD_RW_DDR |= _BV(LCD_RW_BIT);
	LCD_E_DDR |= _BV(LCD_E_BIT);

	//init LCD_DATA
	LCD_DATA_PORT &= ~(0x0F<<LCD_DATA_BIT);

	for (cnt=0;cnt < sizeof(init_sequence);cnt++)
		LCD_byte_out(0,0, init_sequence[cnt]);
}

void LCD_clear(void)
{
	unsigned char cmd;
	//set command
	cmd = 1;
	LCD_byte_out(0, USE_BUSY, cmd);
	cmd = 2;
	LCD_byte_out(0, USE_BUSY, cmd);
}

void LCD_goto(unsigned char row, unsigned char column)
{
	unsigned char addr;
	//set DDRAM Address
	addr = (0x80 | ((row << 6) + column));
	LCD_byte_out(0, USE_BUSY, addr);
}


void LCD_putchar(unsigned char data)
{
	//write data
	LCD_byte_out(LCD_RS, USE_BUSY, data);
}

void LCD_putstring(unsigned char row, unsigned char column, char *data)
{
	unsigned int cnt=0;
	if ((row > LCD_ROW) || (column > LCD_COLUMN))
	{
		row = 0;
		column = 0;
	}
	LCD_goto(row, column);
	while ((data[cnt] != '\0') && (column < LCD_MAXCOLUMN))
	{
		if (data[cnt] == '\n')
		{
			row += 1;
			if (row > LCD_ROW) row = 0;
			column = 0;
			LCD_goto(row, column);
		}
		else LCD_byte_out(LCD_RS, USE_BUSY, data[cnt]);
		column++;
		cnt++;
	}
}
