/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <util/twi.h>
#include <twi_x.h>
#include <twi_eep.h>

unsigned int ee_read_bytes(unsigned char devaddr, unsigned int eeaddr, unsigned int len, unsigned char *buf)
{
	unsigned int iter = 0;
	unsigned int n = 0;
	unsigned char  addrhi, addrlo;
	
	if (eeaddr > EE_SIZE) return 0;
	
	while ((n < len) && (iter < EEP_MAX_ITER))
	{
		addrhi = ((eeaddr+n)>>8) & 0xff;
		addrlo = (eeaddr+n) & 0xff;
		if (TWI_start(TW_START) &&
			TWI_dev_access(devaddr | TW_WRITE) &&
			TWI_write(&addrhi, ACK) &&
			TWI_write(&addrlo, ACK) )

		{
			if (TWI_start(TW_REP_START) &&
				TWI_dev_access(devaddr | TW_READ) )
			{
				do
				{
					if (TWI_read(&buf[n], ACK))
					{
						n++;
					}
					else iter++;
				}
				while ((n < len-1) && (iter < EEP_MAX_ITER));
				if (n == (len-1))
					if (TWI_read(&buf[n], NOACK)) n++;
				TWI_stop();
			}
			else
			{
				iter++;
				TWI_stop();
			}	
		} else {
			iter++;
			TWI_stop();
		}	
	}
	return n;
}


unsigned int ee_write_bytes(unsigned char devaddr, unsigned int eeaddr, unsigned int len, unsigned char *buf)
{
	unsigned int iter = 0;
	unsigned int n = 0;
	unsigned char addrhi, addrlo;
	
	if (eeaddr > EE_SIZE) return 0;
	
	while ((n < len) && (iter < EEP_MAX_ITER))
	{
		addrhi = ((eeaddr+n)>>8) & 0xff;
		addrlo = (eeaddr+n) & 0xff;
		if (TWI_start(TW_START) &&
			TWI_dev_access(devaddr | TW_WRITE) &&
			TWI_write(&addrhi, ACK) &&
			TWI_write(&addrlo, ACK) )
		{
			iter = 0;
			do
			{
				if (TWI_write(&buf[n], ACK))
				{
					n++;
				}
				else iter++;
			}
			while ((((eeaddr+n) % PAGE_SIZE) > 0) && (n < (len-1)) && (iter < EEP_MAX_ITER));
			if (n == (len-1))
				if (TWI_write(&buf[n], ACK)) n++;
			TWI_stop();
		}
		else
		{
			iter++;
			TWI_stop();
		}	
	}
	while (iter < EEP_MAX_ITER)
	{
		if (TWI_start(TW_START) &&
			TWI_dev_access(devaddr | TW_WRITE) &&
			TWI_write(&addrhi, ACK) &&
			TWI_write(&addrlo, ACK) )
		{
			TWI_stop();
			return n;
		}
		else iter++;
	}
	return n;
}
