/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <util/twi.h>
#include <twi_x.h>


#ifndef F_CPU
	#define F_CPU	14745600UL
#endif

void TWI_init(void)
{
	/* initialize TWI clock: 400 kHz clock, TWPS = 0 => prescaler = 1 */
#if defined(TWPS0)
	/* has prescaler (mega128 & newer) */
	TWSR = 0;
#endif
	TWBR = (F_CPU / 400000UL - 16) / 2;
}

unsigned char TWI_action(unsigned char status)
{
	while (!(TWCR & _BV(TWINT)));
	if ((TWSR & 0xF8) == status)
		return 1;
	else
		return 0;
}

unsigned char TWI_start(unsigned char result)
{
	TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);
	return TWI_action(result);
}

unsigned char TWI_stop(void)
{
	TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWSTO);
	return 1;
}

unsigned char TWI_dev_access(unsigned char device_addr)
{
	TWDR = device_addr;
	TWCR = _BV(TWINT) | _BV(TWEN);
	if ((device_addr & 1) == 1)
		return TWI_action(TW_MR_SLA_ACK);
	else
		return TWI_action(TW_MT_SLA_ACK);
}

unsigned char TWI_write(unsigned char *data, unsigned char acknowledge)
{
	TWDR = *data;
	TWCR = _BV(TWINT) | _BV(TWEN);
	if (acknowledge)
		return TWI_action(TW_MT_DATA_ACK);
	else
		return TWI_action(TW_MT_DATA_NACK);
}

unsigned char TWI_read(unsigned char *data, unsigned char acknowledge)
{
	if (acknowledge) {
		TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWEA);
		if (TWI_action(TW_MR_DATA_ACK))
		{
			*data = TWDR;
			return 1;
		}
		else return 0;
	} else {
		TWCR = _BV(TWINT) | _BV(TWEN);
		if (TWI_action(TW_MR_DATA_NACK))
		{
			*data = TWDR;
			return 1;
		}
		else return 0;
	}
}
