/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <util/crc16.h>

#include <lcd-alpha.h>
#include <twi_x.h>
#include <twi_eep.h>
#include <twi_rtc.h>
#include <config.h>
#include <menu.h>
#include <cy22150.h>
#include <1wire/1wire-error.h>
#include <1wire/1wire.h>
#include <1wire/1wire-manage.h>
#include <1wire/1wire-devices.h>


#define TWI_SLA_24CL64	0xa0
#define T0TOP			(unsigned int)((F_CPU * 0.005 / 256) - 1)
#define DIM_PWM_MAX		0x80
#define PRESET_SIZE		256
#define LANGUAGE_MAX	1
//keyboard pins
#define COL_MASK		3
#define COL_PORT		PORTB
#define COL_DDR			DDRB
#define KEY_PIN			PINB
#define KEY_ACT			PB4
//message queue
#define	MSG_PUT			1
#define	MSG_GET			2


char fw_ver[] = "uhu-0.9";


extern pll_settings cy_pll;
extern unsigned char cy22150_buf[256];

struct menuentry cur_menu;
struct menuentry *ptr_menu = NULL;

extern w1_dev *w1_root;
w1_dev *rom, *err_rom;
float w1_t_lpro = 0, w1_t_hs = 0;


_config *config = NULL;
unsigned char *config_raw;

unsigned char errorstat = 0;
volatile unsigned char gray_stat;
volatile unsigned char last_key = 0xFF;
volatile unsigned char pb_delay = 0;

volatile unsigned char remote_cy22150 = 0;
volatile unsigned char remotetimer_5ms = 0;
volatile unsigned int dimtimer_s = 0;
volatile unsigned int emergtimer_5ms = 0;
volatile unsigned char refresh_display = 0;
volatile unsigned char dimvalue = 0;
volatile unsigned char dimupdown = 0;
volatile unsigned int timer_5ms = 0;
volatile unsigned int timer_s = 0;
volatile unsigned int timer_min = 0;

// buffer for messages from rotary encoder and keyboard
volatile unsigned char msg_buf[256];
unsigned char msg_write_ptr = 0;
unsigned char msg_read_ptr = 0;
unsigned char menu_state = 0;
unsigned long long poh;
unsigned long language = LANGUAGE_DEFAULT;
unsigned long contrast = CONTRAST_DEFAULT;
unsigned long dim_on = DIMON_DEFAULT;
unsigned long preset = 1;
unsigned char w1_id_lpro[8], w1_id_hs[8];
unsigned char f_state = 1;
unsigned long f1_input, f2_input;

char LCD_line[LCD_COLUMN+1];
char LCD_emerg[LCD_COLUMN+1];


// function prototypes
void write_config(void);
void dimOn(void);
void dimOff(void);


unsigned char msg_queue(unsigned char action, unsigned char data)
{
	switch (action) {
		case MSG_PUT:
			msg_buf[msg_write_ptr++] = data;
			return data;
			break;
		case MSG_GET:
			if (msg_write_ptr == msg_read_ptr) return 0;
			return msg_buf[msg_read_ptr++];
			break;
	}
	return 0;
}


char gray_bitmap[16] =
{
//	gray_stat =
//	0		1		2		3
	0,		GRAY_L,	GRAY_R,	0,		//gray_new = 0
	GRAY_R,	0,		0,		GRAY_L,	//gray_new = 1
	GRAY_L,	0,		0,		GRAY_R,	//gray_new = 2
	0,		GRAY_R,	GRAY_L,	0,		//gray_new = 3
};


//rotary encoder channel A
ISR (SIG_INTERRUPT0)
{
//	gray_rot += gray_bitmap[(PIND & 0x0C) | gray_stat];
	msg_queue(MSG_PUT, gray_bitmap[(PIND & 0x0C) | gray_stat]);
	gray_stat = (PIND>>2) & 3;
	dimOn();
}

ISR (SIG_INTERRUPT1, ISR_ALIASOF(SIG_INTERRUPT0));

//remote I2C activity
ISR (PCINT0_vect)
{
	//0 -> 1 transition -> exit
	if (PORTA & _BV(PA7)) return;
	//disable INT
	//PCMSK0 &= ~_BV(PCINT7);
	remotetimer_5ms = REMOTETIMEOUT;
	//activate remote USB -> I2C -> CY22150
	PORTC &= ~_BV(PC6);
	//LED remote -> on
	PORTB &= ~_BV(PB7);
	dimOn();
}

//rotary encoder pushbutton
ISR (PCINT1_vect)
{
	//release -> set delay before next accept
	if (PINB & _BV(PB5)) {
		pb_delay = PB_DELAY;
		return;
	}
	dimOn();
	if (pb_delay) return;
	pb_delay = PB_DELAY;
	msg_queue(MSG_PUT, GRAY_PB);
}

unsigned char keymap[16] =
{
//	COL =
//	0			1			2		3
	KEY_RED,	KEY_STAR,	10,		KEY_RAUTE,	//ROW = 0
	KEY_GREEN,	7,			8,		9,			//ROW = 1
	KEY_YELLOW,	4,			5,		6,			//ROW = 2
	KEY_BLUE,	1,			2,		3,			//ROW = 3
};

void keyboard_scan(void)
{
	unsigned char col_cnt;

	for (col_cnt=0;col_cnt<4;col_cnt++) {
		//activate column
		COL_PORT &= ~COL_MASK;
		COL_PORT |= col_cnt;
		asm volatile("nop\n\t""nop\n\t""nop\n\t"::);
		if (! (KEY_PIN & _BV(KEY_ACT))) {
			//key pressed
			if (last_key == keymap[KEY_PIN & 0xF]) return;
			dimOn();
			msg_queue(MSG_PUT, last_key = keymap[KEY_PIN & 0xF]);
			return;
		}
	}
	last_key = 0xFF;
	return;
}

//timer 5ms
ISR (SIG_OUTPUT_COMPARE0A)
{
	if (pb_delay) pb_delay--;

	if (remotetimer_5ms) {
		remotetimer_5ms--;
		if (!remotetimer_5ms) {
			//activate controller I2C -> CY22150
			PORTC |= _BV(PC6);
			//LED remote off
			PORTB |= _BV(PB7);
			//disable INT
			PCMSK0 &= ~_BV(PCINT7);
			remote_cy22150 = 1;
		}
	}

	if (emergtimer_5ms) {
		emergtimer_5ms--;
		if (!emergtimer_5ms) refresh_display = 1;
	}

	if(timer_5ms++ == 200) {
		timer_5ms = 0;
		if (dimtimer_s++ == dim_on) {
			dimtimer_s = 0;
			dimOff();
		}
		poh++;
		if (timer_s++ == 60) {
			timer_s = 0;
			timer_min++;
		}
	}

	if((dimvalue!=0) && (dimvalue!=DIM_PWM_MAX)) {
		if(dimupdown==0) dimvalue--;
		if(dimupdown==1) dimvalue++;
		OCR1B=dimvalue;
	}

	keyboard_scan();
}

void dimOn(void)
{
	dimtimer_s = 0;
	dimupdown = 1;
	if (dimvalue > 0) return;			//already running
	dimvalue = 1;
}

void dimOff(void)
{
	dimupdown = 0;
	if (dimvalue < DIM_PWM_MAX) return;		//already dimming
	dimvalue = DIM_PWM_MAX - 1;
}



void PWM_init(void)
{
	//TIMER0: CTC-mode 5ms timer: TOP=OCR0A=215, prescaler: 256
	TCCR0A = _BV(WGM01);
	TCCR0B = _BV(CS02);
	OCR0A = T0TOP;
	TIMSK0 |= _BV(OCIE0A);

	//TIMER1,OC1A=Kontrast-PWM,OC1B=Backlight-PWM
	//ICR1=TOP
	ICR1 = 0xff;
	//PWM values
	OCR1A = 0x10;
	OCR1B = 0x00;
	//TIMER1: phase correct PWM, TOP=ICR1, prescaler: 1
	TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(WGM11);
	TCCR1B = _BV(WGM13) | _BV(CS10);
	//OC1A, OC1B Pin PWM Output
	DDRD |= _BV(PD4) | _BV(PD5);

	//TIMER2,OC2A=LED-links,OC2B=LED-rechts
	OCR2A = 0x80;
	OCR2B = 0;
	//TIMER2: phase correct PWM, TOP=0xFF, prescaler: 1
	TCCR2A = _BV(COM2A1) | _BV(COM2B1) | _BV(WGM20);
	TCCR2B = _BV(CS20);
	//(OC2A), OC2B Pin PWM Output
	DDRD |= _BV(PD7);
}

void config_defaults(void)
{
	//do not reset power in hours
	//config->poh = 0;
	config->contrast = CONTRAST_DEFAULT;
	contrast = CONTRAST_DEFAULT;
	OCR1A = CONTRAST_DEFAULT;
	config->language = LANGUAGE_DEFAULT;
	language = LANGUAGE_DEFAULT;
	config->dim_on = DIMON_DEFAULT;
	dim_on = DIMON_DEFAULT;
}

unsigned int get_config(unsigned int base_address)
{
	unsigned char cnt;
	unsigned int crc16=0;

	if (!config) {
		config = calloc(1,sizeof(_config));
		config_raw = (unsigned char *)config;
	}

	if ( !ee_read_bytes(TWI_SLA_24CL64, base_address, sizeof(_config), config_raw)) return 0;
	for (cnt=0;cnt<sizeof(_config)-2;cnt++) {
		crc16 = _crc16_update(crc16, config_raw[cnt]);
	}
	if (config->crc16 != crc16) return 0;
	return 1;
}

void read_config(void)
{
	if ( !get_config(CONFIG_BASE_ADDR)) {
		if ( !get_config(CONFIG_BASE_ADDR_ALT)) config_defaults();
	}

	poh = config->poh;
	OCR1A = config->contrast;
	contrast = config->contrast;
	if (config->language > LANGUAGE_MAX) config->language = LANGUAGE_DEFAULT;
	language = config->language;
	dim_on = config->dim_on;

	memcpy(w1_id_lpro, config->w1_id_lpro, sizeof(w1_id_lpro));
	memcpy(w1_id_hs, config->w1_id_hs, sizeof(w1_id_hs));

	write_config();
}

void write_config(void)
{
	unsigned char cnt;
	unsigned int crc16=0;

	if (!config) return;

	config->poh = poh;
	config->contrast = contrast;
	config->language = language;
	config->dim_on = dim_on;
	memcpy(config->w1_id_lpro, w1_id_lpro, sizeof(w1_id_lpro));
	memcpy(config->w1_id_hs, w1_id_hs, sizeof(w1_id_hs));
	//disable TIMER0 INT -> prevent corruption of config->poh
	TIMSK0 &= ~_BV(OCIE0A);
	for (cnt=0;cnt<sizeof(_config)-2;cnt++) {
		crc16 = _crc16_update(crc16, config_raw[cnt]);
	}
	config->crc16 = crc16;
	ee_write_bytes(TWI_SLA_24CL64, CONFIG_BASE_ADDR, sizeof(_config), config_raw);
	ee_write_bytes(TWI_SLA_24CL64, CONFIG_BASE_ADDR_ALT, sizeof(_config), config_raw);
	//enable TIMER0 INT
	TIMSK0 |= _BV(OCIE0A);
}

void load_preset(unsigned long preset_num)
{
	if (preset_num > PRESET_MAX) return;
	ee_read_bytes(TWI_SLA_24CL64, preset_num*PRESET_SIZE, sizeof(cy22150_buf), cy22150_buf);
	rtc_write_bytes(TWI_SLA_CY22150, 0, sizeof(cy22150_buf), cy22150_buf);
}

void save_preset(unsigned long preset_num)
{
	if (preset_num > PRESET_MAX) return;
	rtc_read_bytes(TWI_SLA_CY22150, 0, sizeof(cy22150_buf), cy22150_buf);
	ee_write_bytes(TWI_SLA_24CL64, preset_num*PRESET_SIZE, sizeof(cy22150_buf), cy22150_buf);
}

//delay(ms)
void _delay_long(unsigned int dodelay)
{
	unsigned int cnt;

	for (cnt=0;cnt<dodelay/10;cnt++) {
		_delay_ms(10);
	}
	_delay_ms(dodelay%10);
}

void set_cy_pll(void)
{
	findPLLsettings(&cy_pll);
	OCR2B = 0xFF;
	if (cy_pll.frac < 0) {
		//requested frequency not valid -> LED -> rot
		DDRD |= _BV(PD6);
		OCR2B = 0;
	} else {
		if (cy_pll.frac == 0) {
			//frequency exact -> LED -> gruen
			DDRD |= _BV(PD6);
			OCR2B = 0xFF;
		} else DDRD &= ~_BV(PD6);	//frequency nearly exact -> LED -> off
	}
	if (!checkPLL()) {
		//settings out of range -> LED -> orange
		DDRD |= _BV(PD6);
		if (OCR2B > 0) OCR2B = 0x80;
	}
}

void stat_chk_save(void)
{
	w1_dev *rr_rom;

	//save poh and settings
	write_config();
	
	//read temperature sensors
	if (timer_s & 1) {
		if (rom && ((rom->id[0] == CHIP_DS18B20) || (rom->id[0] == CHIP_DS18S20))) {
			ds18x20_convert(rom->flags&7, rom, rom->flags & DEV_FLAGS_PARASITE_PWR);
			if (rom->next) rom = rom->next; else rom = w1_root;
		}
	} else {
		w1_parasite_power_off(0);
		if ((rr_rom = find_rom(w1_id_lpro))) {
			if ((rr_rom->id[0] == CHIP_DS18B20) || (rr_rom->id[0] == CHIP_DS18S20)) {
				ds18x20_read(rr_rom->flags&7, rr_rom);
				w1_t_lpro = ds18x20_compute(rr_rom);
				if ((w1_t_lpro > 65.0) && (w1_t_lpro != 85.0)) {
					errorstat = ERR_HEAT;
					err_rom = rr_rom;
					emergtimer_5ms = EMERG_TIMEOUT;
				}
			}
		}

		if ((rr_rom = find_rom(w1_id_hs))) {
			if ((rr_rom->id[0] == CHIP_DS18B20) || (rr_rom->id[0] == CHIP_DS18S20)) {
				ds18x20_read(rr_rom->flags&7, rr_rom);
				w1_t_hs = ds18x20_compute(rr_rom);
				if ((w1_t_hs > 60.0) && (w1_t_hs != 85.0)) {
					errorstat = ERR_HEAT;
					err_rom = rr_rom;
					emergtimer_5ms = EMERG_TIMEOUT;
				}
			}
		}
	}

	//display LPRO101 locked and overall status
	if (errorstat) {
		//error red
		OCR2A = 0;
		switch (errorstat) {
			case ERR_HEAT:
				snprintf(LCD_emerg, sizeof(LCD_emerg), "ERR_HEAT%6.2f\337C", (double)ds18x20_compute(err_rom));
				break;
			default:
				snprintf(LCD_emerg, sizeof(LCD_emerg), "ERR_UNKNOWN 0x%02X", errorstat);
				break;
		}
		LCD_putstring(1,0,LCD_emerg);
	} else {
		//display LPRO101 locked status
		if (PINB & _BV(PB6)) OCR2A = 0x80; else OCR2A = 0xFF;
	}
}

void display_frequencies(unsigned char freq_type, unsigned char line)
{
	if (emergtimer_5ms && (line == 1)) {
		LCD_putstring(line,0,LCD_emerg);
		return;
	}
	snprintf(LCD_line, sizeof(LCD_line), "F1:%9lu.%03d",
		(unsigned long)(computefrequency(freq_type) / 1000ULL),
		(unsigned int)(computefrequency(freq_type) % 1000ULL));
	if (freq_type == FTYPE_F2) LCD_line[1] = '2';
	if (freq_type == FTYPE_PLL) {
		LCD_line[0] = 'P';
		LCD_line[1] = 'L';
	}
	LCD_putstring(line,0,LCD_line);
}

void display_menu(void)
{
	unsigned char w1_id_tmp[8];

	static char format_lst[] PROGMEM = "%14S";
	static char format[] PROGMEM = "> %14S";
	static char format_val[] PROGMEM = "%16S";

	if (menu_state == MENU_ST_ONCE) {
		display_frequencies(f_state, 0);
		display_frequencies(FTYPE_PLL, 1);
		return;
	}

	if ((menu_state == MENU_ST_F1) || (menu_state == MENU_ST_F2)) {
		//display actual frequency at line 1
		display_frequencies(f_state, 0);
		snprintf(LCD_line, sizeof(LCD_line), "> ");
	}
	if (menu_state == MENU_ST_VAL) {
		//display menu text at line 1
		snprintf_P(LCD_line, sizeof(LCD_line), format_val, pgm_read_word(cur_menu.menu_text + language));
		LCD_putstring(0,0,LCD_line);
		snprintf(LCD_line, sizeof(LCD_line), "> ");
	}
	if (menu_state == MENU_ST_MENU) {
		//display menu text at line 1
		snprintf_P(LCD_line, sizeof(LCD_line), format, pgm_read_word(cur_menu.menu_text + language));
		LCD_putstring(0,0,LCD_line);
		snprintf(LCD_line, sizeof(LCD_line), "  ");
	}

	if (cur_menu.val) {
		if (cur_menu.val_type & VAL_TYPE_LST) {
			snprintf_P(&LCD_line[2], sizeof(LCD_line) - 2, format_lst, pgm_read_word(cur_menu.menu_text + language + LANG_MAX + (*cur_menu.val << 1)));
		} else {
			//display value at line 2
			switch (cur_menu.val_type & 0x3F) {
				case VAL_TYPE_INT:
					snprintf(&LCD_line[2], sizeof(LCD_line) - 2, "%14d", *(int *)cur_menu.val);
					break;
				case VAL_TYPE_UL:
					snprintf(&LCD_line[2], sizeof(LCD_line) - 2, "%14lu", *cur_menu.val);
					break;
				case VAL_TYPE_TEMP:
					snprintf(&LCD_line[2], sizeof(LCD_line) - 2, "%12.3f\337C", *(double *)cur_menu.val);
					break;
				case VAL_TYPE_STRING:
					snprintf(&LCD_line[2], sizeof(LCD_line) - 2, "%14s", (char *)cur_menu.val);
					break;
				case VAL_TYPE_FREQ:
					snprintf(&LCD_line[2], sizeof(LCD_line) - 2, "%10lu.000", *cur_menu.val);
					break;
				case VAL_TYPE_W1:
					memcpy(w1_id_tmp, (unsigned char *)cur_menu.val, sizeof(w1_id_tmp));
					snprintf(LCD_line, sizeof(LCD_line), "%02X%02X%02X%02X%02X%02X%02X%02X",
						w1_id_tmp[0], w1_id_tmp[1], w1_id_tmp[2], w1_id_tmp[3],
						w1_id_tmp[4], w1_id_tmp[5], w1_id_tmp[6], w1_id_tmp[7]);
					break;
				case VAL_TYPE_POH:
					snprintf(LCD_line, sizeof(LCD_line), "%5ud,%2uh,%2umin",
						(unsigned int)((unsigned long long)*cur_menu.val/86400ULL),
						(unsigned int)(((unsigned long long)*cur_menu.val%86400ULL)/3600ULL),
						(unsigned int)(((unsigned long long)*cur_menu.val%3600ULL)/60ULL));
					break;
			}
		}
	} else snprintf(LCD_line, sizeof(LCD_line), "                ");

	LCD_putstring(1,0,LCD_line);
}


unsigned char menu(void)
{
	extern const struct menuentry freq_menu[];
	unsigned char msg;

	//clear status -> display frequencies once
	if (menu_state == MENU_ST_ONCE) {
		display_menu();
		menu_state = MENU_ST_IDLE;
	}
	msg = msg_queue(MSG_GET, 0);
	if (!msg) return menu_state;

	switch (menu_state) {
		case MENU_ST_IDLE:
			switch (msg) {
				//direkteingabemenue aktivieren
				case KEY_0:
				case KEY_1:
				case KEY_2:
				case KEY_3:
				case KEY_4:
				case KEY_5:
				case KEY_6:
				case KEY_7:
				case KEY_8:
				case KEY_9:
					menu_state = f_state;
					f1_input = 0;
					f2_input = 0;
					if (f_state == 1) ptr_menu = (struct menuentry *)&freq_menu[0];
					if (f_state == 2) ptr_menu = (struct menuentry *)&freq_menu[1];
					memcpy_P(&cur_menu, ptr_menu, sizeof(cur_menu));
					break;
				case GRAY_L:
				case GRAY_R:
				case KEY_BLUE:
					//display alternative divider frequencies
					f_state ^= 3;
					return menu_state = MENU_ST_ONCE;
					break;
				case GRAY_PB:
				case KEY_YELLOW:
					menu_state = MENU_ST_MENU;
					break;
				//all outputs on
				case KEY_GREEN:
					cy22150_buf[0x09] |= 0x3F;
					cy22150_buf_write();
					//write default preset
					ee_write_bytes(TWI_SLA_24CL64, 0*PRESET_SIZE + 0x09, 1, &cy22150_buf[0x09]);
					snprintf(LCD_emerg, sizeof(LCD_emerg), "all outputs ON  ");
					emergtimer_5ms = 400;
					return menu_state = MENU_ST_ONCE;
					break;
				//all outputs off
				case KEY_RED:
					cy22150_buf[0x09] &= 0xC0;
					cy22150_buf_write();
					//write default preset
					ee_write_bytes(TWI_SLA_24CL64, 0*PRESET_SIZE + 0x09, 1, &cy22150_buf[0x09]);
					snprintf(LCD_emerg, sizeof(LCD_emerg), "all outputs OFF ");
					emergtimer_5ms = 400;
					return menu_state = MENU_ST_ONCE;
					break;
			}
			break;
		case MENU_ST_F1:
		case MENU_ST_F2:
			switch (msg) {
				//f1 <--> f2
				case GRAY_L:
				case GRAY_R:
				case KEY_BLUE:
					f_state ^= 3;
					menu_state ^= 3;
					ptr_menu = cur_menu.next;
					memcpy_P(&cur_menu, ptr_menu, sizeof(cur_menu));
					return menu_state;
					break;
				//enter
				case KEY_RAUTE:
				case GRAY_PB:
				case KEY_GREEN:
					if (f_state == 1) {
						cy_pll.f1 = (double)f1_input;	//requested output divider 1
						if (f2_input > 0) {
							cy_pll.f2 = (double)f2_input;	//requested output divider 2
						} else {
							cy_pll.f2 = (double)(computefrequency(FTYPE_F2) / 1000ULL);
						}
						cy_pll.f1_importance = 1;		//wichtigkeit, 1=wichtig
						cy_pll.f2_importance = 0;
					}
					if (f_state == 2) {
						cy_pll.f2 = (double)f2_input;	//requested output divider 2
						if (f1_input > 0) {
							cy_pll.f1 = (double)f1_input;	//requested output divider 1
						} else {
							cy_pll.f1 = (double)(computefrequency(FTYPE_F1) / 1000ULL);
						}
						cy_pll.f1_importance = 0;		//wichtigkeit, 1=wichtig
						cy_pll.f2_importance = 1;
					}
					set_cy_pll();
					cy22150_buf_write();
					//write default preset
					ee_write_bytes(TWI_SLA_24CL64, 0*PRESET_SIZE, sizeof(cy22150_buf), cy22150_buf);
					ptr_menu = NULL;
					snprintf(LCD_emerg, sizeof(LCD_emerg), "iterations %5ud", (unsigned int)cy_pll.iterations);
					emergtimer_5ms = 400;
					return menu_state = MENU_ST_ONCE;
					break;
			}
			break;
		case MENU_ST_MENU:
		case MENU_ST_VAL:
			switch (msg) {
				//enter
				case KEY_RAUTE:
				case GRAY_PB:
				case KEY_GREEN:
					switch (cur_menu.menu_hook) {
						case 9:
							if (menu_state == MENU_ST_VAL) save_preset(preset);
							break;
						case 10:
							if (menu_state == MENU_ST_VAL) load_preset(preset);
							break;
					}
					if (ptr_menu && (cur_menu.val != NULL) && (cur_menu.val_type < 0x80)) menu_state ^= 7;
					break;
			}
			break;
	}

	ptr_menu = do_menuaction(ptr_menu, &cur_menu, msg, &menu_state);
	if (!ptr_menu) return menu_state = MENU_ST_ONCE;
	memcpy_P(&cur_menu, ptr_menu, sizeof(cur_menu));

	return menu_state;
}



int main()
{
	unsigned char cnt, last_stat_save = 0;


	//activate controller I2C -> CY22150
	DDRC |= _BV(PC6);
	PORTC |= _BV(PC6);
	//LED remote off
	DDRB |= _BV(PB7);
	PORTB |= _BV(PB7);
	//keyboard scan bits
	COL_DDR |= COL_MASK;

	TWI_init();
	PWM_init();
	LCD_init();
	cy22150_init();
	
	//pushbutton not pressed, read last settings
	if (PINB & _BV(PB5)) {
		read_config();
		//load default preset;
		load_preset(0);
	} else {
		read_config();
		//pushbutton pressed, load defaults
		config_defaults();
		write_config();
		rtc_read_bytes(TWI_SLA_CY22150, 0, sizeof(cy22150_buf), cy22150_buf);
		//replace all presets with default values
		for (cnt=0;cnt<=PRESET_MAX;cnt++) save_preset(cnt);
		cy22150_init();
	}

	//settings ok -> LED green, out of range -> LED -> orange
	if (checkPLL()) OCR2B = 0xFF; else OCR2B = 0x80;
	DDRD |= _BV(PD6);


	//enable INT0, INT1 -> encoder
	gray_stat = (PIND>>2) & 3;
	EICRA |= _BV(ISC10) | _BV(ISC00);
	EIMSK |= _BV(INT1) | _BV(INT0);

	//enable PCINT13 -> encoder-pushbutton
	PCICR |= _BV(PCIE1);
	PCMSK1 |= _BV(PCINT13);

	//enable PCINT7 -> USB-I2C activity
	PCICR |= _BV(PCIE0);
	PCMSK0 |= _BV(PCINT7);

	//search roms at root segment, master reset
	search_roms(0, 1);
	rom=w1_root;

	sei();

	dimtimer_s = 0;
	dimOn();

	snprintf(LCD_line, sizeof(LCD_line), "Rubidium");
	LCD_putstring(0,0,LCD_line);
	snprintf(LCD_line, sizeof(LCD_line), "Frequenzstandard");
	LCD_putstring(1,0,LCD_line);
	_delay_long(2000);

	//refresh display
	menu_state = MENU_ST_ONCE;

	for (;;) {
		//save config every second
		if (last_stat_save != timer_s) {
			last_stat_save = timer_s;
			stat_chk_save();
		}

		//reread cy22150_buf after remote control
		if (remote_cy22150) {
			remote_cy22150 = 0;
			//refresh buffer and save default preset
			save_preset(0);
			//INT pending?
			if (PCIFR & _BV(PCIE0)) {
				//cancel pending INT
				PCIFR |= _BV(PCIE0);
				//reread cy22150 buffer next cycle
				remote_cy22150 = 1;
			} else {
				//enable INT
				PCMSK0 |= _BV(PCINT7);
			}
			//display frequencies in IDLE state
			if (menu_state == MENU_ST_IDLE) {
				menu_state = MENU_ST_ONCE;
			}
			//settings ok -> LED green, out of range -> LED -> orange
			if (checkPLL()) OCR2B = 0xFF; else OCR2B = 0x80;
			DDRD |= _BV(PD6);
			//dummy message to refresh display
			msg_queue(MSG_PUT, KEY_DUMMY);
		}

		if (refresh_display) {
			refresh_display = 0;
			if (menu_state == MENU_ST_IDLE) menu_state = MENU_ST_ONCE;
		}

		if ((menu_state = menu())) display_menu();
	}
	return 0;
}
