/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2010 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of Rubidium-ClockDistribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * AVR Studio 4.15
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2010 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <string.h>
#include <avr/pgmspace.h>

#include <1wire/1wire-error.h>
#include <1wire/1wire.h>
#include <1wire/1wire-manage.h>

#include <config.h>
#include <menu.h>

#include <cy22150.h>


extern char fw_ver[];
extern float w1_t_lpro, w1_t_hs;
extern unsigned long long poh;
extern unsigned long language;
extern unsigned long contrast;
extern unsigned long dim_on;
extern unsigned long f1_input;
extern unsigned long f2_input;
extern unsigned long preset;
extern pll_settings cy_pll;
extern unsigned char w1_id_lpro[8], w1_id_hs[8];
extern w1_dev *w1_root;

int out_src;

//forward declarations
const struct menuentry root_menu[];
const struct menuentry preset_menu[];
const struct menuentry setup_menu[];
const struct menuentry info_menu[];
const struct menuentry pll_menu[];
const struct menuentry output_menu[];

//menu strings
static char de_text_001[] PROGMEM = "Speicher";
static char de_text_002[] PROGMEM = "PLL";
static char de_text_003[] PROGMEM = "Einstellungen";
static char de_text_004[] PROGMEM = "Ende";
static char de_text_005[] PROGMEM = "Sichern";
static char de_text_006[] PROGMEM = "Laden";
static char de_text_007[] PROGMEM = "Eingabe >>>";
static char de_text_008[] PROGMEM = "LCD-Kontrast";
static char de_text_009[] PROGMEM = "Sprache";
static char de_text_010[] PROGMEM = "Licht-An-Dauer";
static char de_text_011[] PROGMEM = "Direkteigabe F1>";
static char de_text_012[] PROGMEM = "Direkteigabe F2>";
static char de_text_013[] PROGMEM = "Informationen";
static char de_text_014[] PROGMEM = "Software Ver.";
static char de_text_015[] PROGMEM = "LPRO-101 Temp.";
static char de_text_016[] PROGMEM = "K\365hlk\357rper Tmp";
static char de_text_017[] PROGMEM = "PLL-Faktor P";
static char de_text_018[] PROGMEM = "PLL-Faktor Q";
static char de_text_019[] PROGMEM = "PLL-Divider D1";
static char de_text_020[] PROGMEM = "PLL-Divider D2";
static char de_text_021[] PROGMEM = "Ausg\341nge";
static char de_text_022[] PROGMEM = "LCLK1";
static char de_text_023[] PROGMEM = "LCLK2";
static char de_text_024[] PROGMEM = "LCLK3";
static char de_text_025[] PROGMEM = "LCLK4";
static char de_text_026[] PROGMEM = "CLK5";
static char de_text_027[] PROGMEM = "CLK6";
static char de_text_028[] PROGMEM = "Deutsch";
static char de_text_029[] PROGMEM = "englisch";
static char de_text_030[] PROGMEM = "F ref";
static char de_text_031[] PROGMEM = "DIV1CLK/DIV1N";
static char de_text_032[] PROGMEM = "DIV1CLK/2";
static char de_text_033[] PROGMEM = "DIV1CLK/3";
static char de_text_034[] PROGMEM = "DIV2CLK/DIV2N";
static char de_text_035[] PROGMEM = "DIV2CLK/2";
static char de_text_036[] PROGMEM = "DIV2CLK/4";
static char de_text_037[] PROGMEM = "UNDEFINED!";
static char de_text_038[] PROGMEM = "OFF";
static char de_text_039[] PROGMEM = "LPRO-101 w1-ID";
static char de_text_040[] PROGMEM = "K\365hlk. w1-ID";
static char de_text_041[] PROGMEM = "Betriebsstd.";


static char en_text_001[] PROGMEM = "presets";
static char en_text_002[] PROGMEM = "PLL";
static char en_text_003[] PROGMEM = "settings";
static char en_text_004[] PROGMEM = "exit";
static char en_text_005[] PROGMEM = "save";
static char en_text_006[] PROGMEM = "recall";
static char en_text_007[] PROGMEM = "input >>>";
static char en_text_008[] PROGMEM = "LCD-contrast";
static char en_text_009[] PROGMEM = "language";
static char en_text_010[] PROGMEM = "dim-delay";
static char en_text_011[] PROGMEM = "direct input F1>";
static char en_text_012[] PROGMEM = "direct input F2>";
static char en_text_013[] PROGMEM = "information";
static char en_text_014[] PROGMEM = "firmware ver.";
static char en_text_015[] PROGMEM = "LPRO-101 temp.";
static char en_text_016[] PROGMEM = "heatsink temp.";
static char en_text_017[] PROGMEM = "PLL-factor P";
static char en_text_018[] PROGMEM = "PLL-factor Q";
static char en_text_019[] PROGMEM = "PLL-divider D1";
static char en_text_020[] PROGMEM = "PLL-divider D2";
static char en_text_021[] PROGMEM = "output source";
static char en_text_028[] PROGMEM = "german";
static char en_text_029[] PROGMEM = "english";
static char en_text_040[] PROGMEM = "heatsink w1-ID";
static char en_text_041[] PROGMEM = "power on hours";

//menu string pointer
static char *text_001[] PROGMEM = { de_text_001, en_text_001 };
static char *text_002[] PROGMEM = { de_text_002, en_text_002 };
static char *text_003[] PROGMEM = { de_text_003, en_text_003 };
static char *text_004[] PROGMEM = { de_text_004, en_text_004 };
static char *text_005[] PROGMEM = { de_text_005, en_text_005 };
static char *text_006[] PROGMEM = { de_text_006, en_text_006 };
static char *text_007[] PROGMEM = { de_text_007, en_text_007 };
static char *text_008[] PROGMEM = { de_text_008, en_text_008 };
static char *text_009[] PROGMEM = { de_text_009, en_text_009, de_text_028, en_text_028, de_text_029, en_text_029 };
static char *text_010[] PROGMEM = { de_text_010, en_text_010 };
static char *text_011[] PROGMEM = { de_text_011, en_text_011 };
static char *text_012[] PROGMEM = { de_text_012, en_text_012 };
static char *text_013[] PROGMEM = { de_text_013, en_text_013 };
static char *text_014[] PROGMEM = { de_text_014, en_text_014 };
static char *text_015[] PROGMEM = { de_text_015, en_text_015 };
static char *text_016[] PROGMEM = { de_text_016, en_text_016 };
static char *text_017[] PROGMEM = { de_text_017, en_text_017 };
static char *text_018[] PROGMEM = { de_text_018, en_text_018 };
static char *text_019[] PROGMEM = { de_text_019, en_text_019 };
static char *text_020[] PROGMEM = { de_text_020, en_text_020 };
static char *text_021[] PROGMEM = { de_text_021, en_text_021 };

static char *text_022[] PROGMEM = { de_text_022, de_text_022,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };
static char *text_023[] PROGMEM = { de_text_023, de_text_023,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };
static char *text_024[] PROGMEM = { de_text_024, de_text_024,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };
static char *text_025[] PROGMEM = { de_text_025, de_text_025,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };
static char *text_026[] PROGMEM = { de_text_026, de_text_026,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };
static char *text_027[] PROGMEM = { de_text_027, de_text_027,
	de_text_030, de_text_030, de_text_031, de_text_031, de_text_032, de_text_032, de_text_033, de_text_033,
	de_text_034, de_text_034, de_text_035, de_text_035, de_text_036, de_text_036, de_text_037, de_text_037, de_text_038, de_text_038 };

static char *text_028[] PROGMEM = { de_text_028, en_text_028 };
static char *text_029[] PROGMEM = { de_text_029, en_text_029 };
static char *text_039[] PROGMEM = { de_text_039, de_text_039 };
static char *text_040[] PROGMEM = { de_text_040, en_text_040 };
static char *text_041[] PROGMEM = { de_text_041, en_text_041 };




const struct menuentry freq_menu[] PROGMEM =
{
	{ text_011, NULL                               , (struct menuentry *)&freq_menu[1], (struct menuentry *)&freq_menu[1], &f1_input, 0, 200000000, VAL_TYPE_FREQ, 0 },
	{ text_012, NULL                               , (struct menuentry *)&freq_menu[0], (struct menuentry *)&freq_menu[0], &f2_input, 0, 200000000, VAL_TYPE_FREQ, 0 }
};

const struct menuentry root_menu[] PROGMEM =
{
	{ text_001, (struct menuentry *)&preset_menu[0], (struct menuentry *)&root_menu[5], (struct menuentry *)&root_menu[1], NULL, 0, 0, 0, 0 },
	{ text_002, (struct menuentry *)&pll_menu[0]   , (struct menuentry *)&root_menu[0], (struct menuentry *)&root_menu[2], NULL, 0, 0, 0, 0 },
	{ text_021, (struct menuentry *)&output_menu[0], (struct menuentry *)&root_menu[1], (struct menuentry *)&root_menu[3], NULL, 0, 0, 0, 0 },
	{ text_003, (struct menuentry *)&setup_menu[0] , (struct menuentry *)&root_menu[2], (struct menuentry *)&root_menu[4], NULL, 0, 0, 0, 0 },
	{ text_013, (struct menuentry *)&info_menu[0]  , (struct menuentry *)&root_menu[3], (struct menuentry *)&root_menu[5], NULL, 0, 0, 0, 0 },
	{ text_004, NULL                               , (struct menuentry *)&root_menu[4], (struct menuentry *)&root_menu[0], NULL, 0, 0, 0, 0 }
};

const struct menuentry preset_menu[] PROGMEM =
{
	{ text_005, (struct menuentry *)&preset_menu[0], (struct menuentry *)&preset_menu[2], (struct menuentry *)&preset_menu[1], &preset, 1, PRESET_MAX, VAL_TYPE_UL, 9  },
	{ text_006, (struct menuentry *)&preset_menu[1], (struct menuentry *)&preset_menu[0], (struct menuentry *)&preset_menu[2], &preset, 1, PRESET_MAX, VAL_TYPE_UL, 10 },
	{ text_004, (struct menuentry *)&root_menu[0]  , (struct menuentry *)&preset_menu[1], (struct menuentry *)&preset_menu[0], NULL   , 0, 0         , 0          , 0  },
};

const struct menuentry setup_menu[] PROGMEM =
{
	{ text_008, (struct menuentry *)&setup_menu[0] , (struct menuentry *)&setup_menu[5], (struct menuentry *)&setup_menu[1], &contrast                 , 0, 255  , VAL_TYPE_UL               , 1 },
	{ text_009, (struct menuentry *)&setup_menu[1] , (struct menuentry *)&setup_menu[0], (struct menuentry *)&setup_menu[2], &language                 , 0, 1    , VAL_TYPE_UL | VAL_TYPE_LST, 0 },
	{ text_010, (struct menuentry *)&setup_menu[2] , (struct menuentry *)&setup_menu[1], (struct menuentry *)&setup_menu[3], &dim_on                   , 0, 65535, VAL_TYPE_UL               , 0 },
	{ text_039, (struct menuentry *)&setup_menu[3] , (struct menuentry *)&setup_menu[2], (struct menuentry *)&setup_menu[4], (unsigned long*)w1_id_lpro, 0, 65535, VAL_TYPE_W1               , 0 },
	{ text_040, (struct menuentry *)&setup_menu[4] , (struct menuentry *)&setup_menu[3], (struct menuentry *)&setup_menu[5], (unsigned long*)w1_id_hs  , 0, 65535, VAL_TYPE_W1               , 0 },
	{ text_004, (struct menuentry *)&root_menu[3]  , (struct menuentry *)&setup_menu[4], (struct menuentry *)&setup_menu[0], NULL                      , 0, 0    , 0                         , 0 },
};

const struct menuentry info_menu[] PROGMEM =
{
	{ text_014, (struct menuentry *)&root_menu[4]  , (struct menuentry *)&info_menu[4] , (struct menuentry *)&info_menu[1] , (unsigned long*)&fw_ver   , 0, 0, VAL_TYPE_STRING | VAL_TYPE_RO , 0 },
	{ text_015, (struct menuentry *)&root_menu[4]  , (struct menuentry *)&info_menu[0] , (struct menuentry *)&info_menu[2] , (unsigned long*)&w1_t_lpro, 0, 0, VAL_TYPE_TEMP | VAL_TYPE_RO   , 0 },
	{ text_016, (struct menuentry *)&root_menu[4]  , (struct menuentry *)&info_menu[1] , (struct menuentry *)&info_menu[3] , (unsigned long*)&w1_t_hs  , 0, 0, VAL_TYPE_TEMP | VAL_TYPE_RO   , 0 },
	{ text_041, (struct menuentry *)&root_menu[4]  , (struct menuentry *)&info_menu[2] , (struct menuentry *)&info_menu[4] , (unsigned long*)&poh      , 0, 0, VAL_TYPE_POH | VAL_TYPE_RO    , 0 },
	{ text_004, (struct menuentry *)&root_menu[4]  , (struct menuentry *)&info_menu[3] , (struct menuentry *)&info_menu[0] , NULL                      , 0, 0, 0                             , 0 },
};

const struct menuentry pll_menu[] PROGMEM =
{
	{ text_017, (struct menuentry *)&pll_menu[0]   , (struct menuentry *)&pll_menu[4]  , (struct menuentry *)&pll_menu[1]  , (unsigned long*)&cy_pll.p , 8, 2055, VAL_TYPE_INT, 8 },
	{ text_018, (struct menuentry *)&pll_menu[1]   , (struct menuentry *)&pll_menu[0]  , (struct menuentry *)&pll_menu[2]  , (unsigned long*)&cy_pll.q , 2, 129 , VAL_TYPE_INT, 8 },
	{ text_019, (struct menuentry *)&pll_menu[2]   , (struct menuentry *)&pll_menu[1]  , (struct menuentry *)&pll_menu[3]  , (unsigned long*)&cy_pll.d1, 2, 127 , VAL_TYPE_INT, 8 },
	{ text_020, (struct menuentry *)&pll_menu[3]   , (struct menuentry *)&pll_menu[2]  , (struct menuentry *)&pll_menu[4]  , (unsigned long*)&cy_pll.d2, 2, 127 , VAL_TYPE_INT, 8 },
	{ text_004, (struct menuentry *)&root_menu[1]  , (struct menuentry *)&pll_menu[3]  , (struct menuentry *)&pll_menu[0]  , NULL                      , 0, 0   , 0           , 0 },
};

const struct menuentry output_menu[] PROGMEM =
{
	{ text_022, (struct menuentry *)&output_menu[0], (struct menuentry *)&output_menu[6]  , (struct menuentry *)&output_menu[1]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 2 },
	{ text_023, (struct menuentry *)&output_menu[1], (struct menuentry *)&output_menu[0]  , (struct menuentry *)&output_menu[2]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 3 },
	{ text_024, (struct menuentry *)&output_menu[2], (struct menuentry *)&output_menu[1]  , (struct menuentry *)&output_menu[3]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 4 },
	{ text_025, (struct menuentry *)&output_menu[3], (struct menuentry *)&output_menu[2]  , (struct menuentry *)&output_menu[4]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 5 },
	{ text_026, (struct menuentry *)&output_menu[4], (struct menuentry *)&output_menu[3]  , (struct menuentry *)&output_menu[5]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 6 },
	{ text_027, (struct menuentry *)&output_menu[5], (struct menuentry *)&output_menu[4]  , (struct menuentry *)&output_menu[6]  , (unsigned long*)&out_src , 0, 8, VAL_TYPE_INT | VAL_TYPE_LST , 7 },
	{ text_004, (struct menuentry *)&root_menu[2]  , (struct menuentry *)&output_menu[5]  , (struct menuentry *)&output_menu[0]  , NULL                     , 0, 0, 0           , 0 },
};









struct menuentry *do_menuaction(struct menuentry *m_ptr, struct menuentry *m_entry, unsigned char msg, unsigned char *menustat)
{
	unsigned char cnt;
	w1_dev *w1_actrom;
	
	//default -> stay in actual menu
	struct menuentry *rv = m_ptr;

	//first action -> return root menu
	if (m_ptr == NULL) return (struct menuentry *)&root_menu[0];

	//no action -> stay in actual menu
	if (!msg) return m_ptr;
	
	if (*menustat == MENU_ST_MENU) {
		switch (msg) {
			case KEY_RED:
				*menustat = MENU_ST_IDLE;
				return NULL;
				break;
			case GRAY_L:
				rv = m_entry->prev;
				memcpy_P(m_entry, rv, sizeof(*m_entry));
				break;
			case GRAY_R:
			case KEY_STAR:
			case KEY_YELLOW:
				rv = m_entry->next;
				memcpy_P(m_entry, rv, sizeof(*m_entry));
				break;
			case GRAY_PB:
			case KEY_GREEN:
			case KEY_RAUTE:
				rv = m_entry->enter;
				memcpy_P(m_entry, rv, sizeof(*m_entry));
				break;
		}

		switch (m_entry->menu_hook) {
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				getOutput(m_entry->menu_hook - 2, &out_src);
				break;
			case 8:
				getPQD1D2(&cy_pll);
				break;
		}
	}

	if ((*menustat == MENU_ST_VAL) || (*menustat == MENU_ST_F1) || (*menustat == MENU_ST_F2)) {
		//read only value -> exit
		if (m_entry->val_type & VAL_TYPE_RO) return rv;

		switch (m_entry->menu_hook) {
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				getOutput(m_entry->menu_hook - 2, &out_src);
				break;
			case 8:
				if ((int)*m_entry->val >= m_entry->min) {
					getPQD1D2(&cy_pll);
				}
				break;
		}

		switch (msg) {
			case KEY_RED:
				*menustat = MENU_ST_IDLE;
				return NULL;
				break;
			case KEY_1:
			case KEY_2:
			case KEY_3:
			case KEY_4:
			case KEY_5:
			case KEY_6:
			case KEY_7:
			case KEY_8:
			case KEY_9:
			case KEY_0:
				switch (m_entry->val_type & 0x3F) {
					case VAL_TYPE_UL:
					case VAL_TYPE_FREQ:
						*m_entry->val = *m_entry->val * 10 + msg % 10;
						if (*m_entry->val > m_entry->max) *m_entry->val = *m_entry->val / 10;
						break;
					case VAL_TYPE_INT:
						if (m_entry->max < 10) {
							*m_entry->val = (int)msg % (int)10;
							if ((int)*m_entry->val > m_entry->max) *m_entry->val = (int)m_entry->max;
						} else {
							*m_entry->val = (int)*m_entry->val * (int)10 + (int)msg % (int)10;
							if ((int)*m_entry->val > m_entry->max) *m_entry->val = (int)*m_entry->val / (int)10;
						}
						break;
					case VAL_TYPE_W1:
						w1_actrom = w1_root;
						for (cnt=0;cnt<msg%10;cnt++) {
							if (w1_actrom->next) w1_actrom = w1_actrom->next;
						}
						if (w1_actrom) memcpy((unsigned char*)m_entry->val, w1_actrom->id, 8);
						break;
				}
				break;
			case GRAY_L:
				switch (m_entry->val_type & 0x3F) {
					case VAL_TYPE_UL:
					case VAL_TYPE_FREQ:
						if (*m_entry->val > 0) *m_entry->val -= 1;
						if (*m_entry->val < m_entry->min) *m_entry->val = m_entry->min;
						break;
					case VAL_TYPE_INT:
						if ((int)*m_entry->val > 0) *m_entry->val -= (int)1;
						if ((int)*m_entry->val < m_entry->min) *m_entry->val = (int)m_entry->min;
						break;
					case VAL_TYPE_W1:
						if ((w1_actrom = find_rom((unsigned char*)m_entry->val))) {
							if (w1_actrom->next) {
								w1_actrom = w1_actrom->next;
							} else {
								w1_actrom = w1_root;
							}
						} else {
							w1_actrom = w1_root;
						}
						if (w1_actrom) memcpy((unsigned char*)m_entry->val, w1_actrom->id, 8);
						break;
				}
				break;
			case GRAY_R:
				switch (m_entry->val_type & 0x3F) {
					case VAL_TYPE_UL:
					case VAL_TYPE_FREQ:
						*m_entry->val += 1;
						if (*m_entry->val > m_entry->max) *m_entry->val = m_entry->max;
						break;
					case VAL_TYPE_INT:
						*m_entry->val += (int)1;
						if ((int)*m_entry->val > m_entry->max) *m_entry->val = (int)m_entry->max;
						break;
					case VAL_TYPE_W1:
						if ((w1_actrom = find_rom((unsigned char*)m_entry->val))) {
							if (w1_actrom->next) {
								w1_actrom = w1_actrom->next;
							} else {
								w1_actrom = w1_root;
							}
						} else {
							w1_actrom = w1_root;
						}
						if (w1_actrom) memcpy((unsigned char*)m_entry->val, w1_actrom->id, 8);
						break;
				}
				break;
			case KEY_STAR:
				switch (m_entry->val_type & 0x3F) {
					case VAL_TYPE_UL:
					case VAL_TYPE_FREQ:
						*m_entry->val = *m_entry->val / 10;
						break;
					case VAL_TYPE_INT:
						*m_entry->val = (int)*m_entry->val / (int)10;
						break;
				}
				break;
		}

		switch (m_entry->menu_hook) {
			case 1:
				OCR1A = (unsigned char)*m_entry->val;
				break;
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
				setOutput(m_entry->menu_hook - 2, &out_src);
				cy22150_buf_write();
				break;
			case 8:
				if ((int)*m_entry->val >= m_entry->min) {
					cy_pll.frac = 0;
					setPQD1D2(&cy_pll, 0);
					setChargePump(&cy_pll);
					cy22150_buf_write();
				}
				break;
		}
	}

	return rv;
}
